ONE_HUNDRED: int = 100

SEGMENT_MATCH_OPERATOR = "segmentMatch"
IN_OPERATOR = "in"
EQUAL_OPERATOR = "equal"
GT_OPERATOR = "gt"
STARTS_WITH_OPERATOR = "starts_with"
ENDS_WITH_OPERATOR = "ends_with"
CONTAINS_OPERATOR = "contains"
EQUAL_SENSITIVE_OPERATOR = "equal_sensitive"
